(function(){

	window.$FS_FUNCTION_CACHE = {};

	window.$FS_FUNCTION_ADD = function(object) {
		var id = "func_" + (new Date()).getTime() + "_" + Math.random();
		window.$FS_FUNCTION_CACHE[id] = object;
		return id;
	};

	window.$FS_FUNCTION_EXEC = function(id, parameters) {
		var func = window.$FS_FUNCTION_CACHE[id];
		if (func) {
			window.$FS_FUNCTION_CACHE[id] = null;
			func.apply(func, parameters ? parameters : []);
		}
	};

	window.$FS_TYPEOF = function(object) {
		if (object != null) {
			var type = typeof object;
			if (type !== "undefined") {
				if (type === "object") {
					//noinspection JSUnresolvedVariable,JSUnresolvedFunction
					if (object instanceof Array || !!Array.isArray && Array.isArray(object)) {
						// instanceof Array might not work in cross-frame environments, so we need to call isArray method as well (not supported by all browser versions)
						return "array";
					} else if (object instanceof Date || Object.prototype.toString.call(object) === "[object Date]") {
						// Object.prototype.toString.call: unfortunately there is no isDate method to check type in a cross-frame environment :\
						return "date";
					}
				}
				return type;
			}
		}
		return 'null';
	};

	window.$FS_XML_NORMALIZE = function(string) {
		string = string.replace(new RegExp('&', 'g'), '&amp;');
		string = string.replace(new RegExp('<', 'g'), '&lt;');
		string = string.replace(new RegExp('>', 'g'), '&gt;');
		return string;
	};

	window.$FS_XML_NORMALIZE_ATTR = function(string) {
		string = window.$FS_XML_NORMALIZE(string);
		string = string.replace(new RegExp('"', 'g'), '&quot;');
		return string;
	};

	window.$FS_JSO2STR = function(object) {
		var xml = [], type = window.$FS_TYPEOF(object);
		xml[xml.length] = '<' + type + '>';
		if (type === 'object') {
			for (var key in object) {
				if (object.hasOwnProperty(key)) {
					xml[xml.length] = '<entry key="' + window.$FS_XML_NORMALIZE_ATTR(key) + '">';
					xml[xml.length] = window.$FS_JSO2STR(object[key]);
					xml[xml.length] = '</entry>';
				}
			}
		} else if (type === 'array') {
			for (var i = 0; i < object.length; i++) {
				xml[xml.length] = window.$FS_JSO2STR(object[i]);
			}
		} else if (type === 'date') {
			xml[xml.length] = '' + ((object && object.getTime) ? object.getTime() : 0);
		} else if (type === 'string') {
			xml[xml.length] = window.$FS_XML_NORMALIZE(object);
		} else if (type === "function") {
			xml[xml.length] = window.$FS_FUNCTION_ADD(object);
		} else {
			xml[xml.length] = '' + object;
		}
		xml[xml.length] = '</' + type + '>';
		return xml.join('');
	};

	window.$FS_INVOKE_J = function(uuid, parameter) {
		if (window.$FS_INVOKER) {
			var result = window.$FS_INVOKER.invoke(uuid, window.$FS_JSO2STR(parameter));
			if (typeof result == 'string') {
				result = eval(result);
			}
			return result;
		}
		return undefined;
	};

	window.$FS_CREATE_PROXY = function(uuid) {
		function func() {
			var parameter = [];
			for (var i = 0; i < func.arguments.length; i++){
				parameter[i] = func.arguments[i];
			}
			return window.$FS_INVOKE_J.call(this, uuid, parameter);
		}
		return func;
	};

})();
