<!--suppress CheckTagEmptyBody -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <xsl:output method="html" indent="yes" encoding="UTF-8"/>
    <xsl:decimal-format decimal-separator="." grouping-separator=","/>

    <xsl:template match="REPORT">
        <html>
            <head>
                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
                <style type="text/css">
                    body {
                    font-family: helvetica, verdana, arial;
                    color: #505050;
                    font-size: 12px;
                    margin-top: 0px;
                    }

                    h1 {
                    color: #606060;
                    font-size: 22px;
                    }

                    h2 {
                    margin-top: 35px;
                    color: #7F1345;
                    font-size: 16px;
                    }

                    h3 {
                    color: #606060;
                    margin-left: 20px;
                    font-size: 14px;
                    }

                    h4 {
                    margin-left: 20px;
                    margin-bottom: 10px;
                    }

                    div.stacktrace {
                    margin-left: 20px;
                    margin-bottom: 30px;
                    font-size: 11px;
                    }

                    ul, table {
                    margin-left: 20px;
                    margin-bottom: 10px;
                    padding: 0;
                    list-style-type: none;
                    }

                    td.left {
                    width: 160px;
                    vertical-align: top;
                    background-color: #eeeeee;
                    }

                    div.toc {
                    background-color: #eeeeee;
                    margin-top: 35px;
                    padding: 15px;
                    padding-left: 5px;
                    }

                    div.text {
                    width: 600px;
                    margin-left: 20px;
                    }

                    a.ref {
                    margin-left: 15px;
                    color: #7F1345;
                    }

                    a.subref {
                    margin-left: 35px;
                    color: #606060;
                    }

                    div.actions {
                    color: #606060;
                    text-align: right;
                    margin-top: 20px;
                    margin-right: 50px;
                    margin-bottom: -15px;
                    }

                    div.actions a {
                    color: #8c8c8c;
                    font-weight: bold;
                    margin: 3px;
                    }

                    table.logo {
                    border-spacing: 0px;
                    border-collapse: collapse; /* IE7 */
                    float: right;
                    margin-right: 50px;
                    margin-top: 0px;
                    }

                    table.logo tr {
                    height: 10px;
                    padding: 0px;
                    }

                    table.logo tr td {
                    width: 10px;
                    padding: 0px;
                    }

                    blank {
                    height: 10px;
                    width: 100%;
                    }

                    toggleable {
                    text-decoration: underline;
                    font-size: 17px;
                    }
                </style>
                <script type="text/javascript" language="javascript">
                    function toggleDisplay(element) {
                    if(element.style.display == "none") {
                    element.style.display = "inline"
                    } else {
                    element.style.display = "none"
                    }
                    }
                </script>

            </head>
            <body onload="isReady=true">
                <table class="logo">
                    <tr>
                        <td style="background-color:#d49fb3;"></td>
                        <td style="background-color:#c87e99;"></td>
                        <td style="background-color:#b8547a;"></td>
                        <td style="background-color:#aa2e64;"></td>
                        <td style="background-color:#9a1155;"></td>
                    </tr>
                    <tr>
                        <td style="background-color:#8c124d;"></td>
                        <td style="background-color:#b8547a;"></td>
                        <td style="background-color:#aa2e64;"></td>
                        <td style="background-color:#9a1155;"></td>
                        <td style="background-color:#7a1245;"></td>
                    </tr>
                    <tr>
                        <td style="background-color:#b8547a;"></td>
                        <td style="background-color:#aa2e64;"></td>
                        <td style="background-color:#b8547a;"></td>
                        <td style="background-color:#c87e99;"></td>
                        <td style="background-color:#7a1245;"></td>
                    </tr>
                    <tr>
                        <td style="background-color:#89124c;"></td>
                        <td style="background-color:#9a1155;"></td>
                        <td style="background-color:#c87e99;"></td>
                        <td style="background-color:#d49fb3;"></td>
                        <td style="background-color:#6f143f;"></td>
                    </tr>
                    <tr>
                        <td style="background-color:#9a1155;"></td>
                        <td style="background-color:#aa2e64;"></td>
                        <td style="background-color:#7a1245;"></td>
                        <td style="background-color:#6f143f;"></td>
                        <td style="background-color:#631139;"></td>
                    </tr>
                </table>
                <br/>
                <div class='actions'>
                    <a href='javascript:print();'>print</a>
                </div>
                <h1 style="display:inline; font-family:times new roman, serife;font-weight:normal;color: #7F1345;">
                    First
                </h1>
                <h1 style="display:inline;font-family:times new roman, serife;font-weight:bold;color:#474747;">
                    Spirit
                </h1>
                <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
                <h1 style="display:inline; font-size:19px;color:#8c8c8c;">
                    <xsl:value-of select="TITLE"/>
                </h1>
                <br/>
                <br/>
                <br/>

                <xsl:call-template name="data.abstract"/>
                <xsl:call-template name="data.toc"/>
                <xsl:call-template name="data.section"/>
                <xsl:call-template name="data.footer"/>
                <xsl:call-template name="data.raw"/>

            </body>
        </html>
    </xsl:template>

    <xsl:template match="INFO" name="data.abstract">
        <div class="text" style="color:#8c8c8c;">
            <xsl:value-of select="INFO/TIMESTAMP"/>
            <br/>
            User:
            <xsl:value-of select="INFO/USER"/>
            <br/>
            <br/>
            <b>
                <xsl:value-of select="INFO/COMMENT/attribute::name"/>
            </b>
            <pre>
                <xsl:value-of select="INFO/COMMENT"/>
            </pre>
        </div>
    </xsl:template>

    <xsl:template match="TOC" name="data.toc">
        <div class="toc">
            <h2 style="margin-top: 0px">
                <xsl:value-of select="TOC/attribute::name"/>
            </h2>
            <xsl:for-each select="TOC/SECTION">
                <xsl:element name="a">
                    <xsl:attribute name="href">#<xsl:value-of select="attribute::name"/>
                    </xsl:attribute>
                    <xsl:attribute name="class">ref</xsl:attribute>
                    <xsl:value-of select="attribute::name"/>
                </xsl:element>
                <br/>

                <xsl:for-each select="SUBSECTION">
                    <xsl:element name="a">
                        <xsl:attribute name="href">#<xsl:value-of select="attribute::name"/>
                        </xsl:attribute>
                        <xsl:attribute name="class">subref</xsl:attribute>
                        <xsl:value-of select="attribute::name"/>
                    </xsl:element>
                    <br/>
                </xsl:for-each>

                <xsl:for-each select="ENTRIES/ENTRY">
                    <xsl:element name="a">
                        <xsl:attribute name="href">#stack<xsl:value-of select="position()"/>
                        </xsl:attribute>
                        <xsl:attribute name="class">subref</xsl:attribute>
                        <xsl:value-of select="substring(attribute::name, 1, 250)"/>
                    </xsl:element>
                    <br/>
                </xsl:for-each>

                <xsl:for-each select="THREADDUMPS/THREADDUMP">
                    <xsl:element name="a">
                        <xsl:attribute name="href">#threaddump<xsl:value-of select="position()"/>
                        </xsl:attribute>
                        <xsl:attribute name="class">subref</xsl:attribute>
                        <xsl:value-of select="substring(attribute::name, 1, 250)"/>
                    </xsl:element>
                    <br/>
                </xsl:for-each>
            </xsl:for-each>
        </div>
    </xsl:template>

    <xsl:template match="SECTION" name="data.section">
        <xsl:for-each select="SECTION">
            <h2>
                <xsl:element name="a">
                    <xsl:attribute name="name">
                        <xsl:value-of select="attribute::name"/>
                    </xsl:attribute>
                </xsl:element>
                <xsl:value-of select="attribute::name"/>
            </h2>

            <xsl:call-template name="data.subsection.title"/>

            <xsl:if test="@id = 'ExceptionalEvents'">
                <xsl:call-template name="data.stacktrace"/>
            </xsl:if>

            <xsl:if test="@id = 'ThreadDump'">
                <xsl:call-template name="data.threaddump"/>
            </xsl:if>

        </xsl:for-each>
    </xsl:template>


    <xsl:template name="data.subsection.title">
        <xsl:for-each select="SUBSECTION">
            <h3>
                <xsl:element name="a">
                    <xsl:attribute name="name">
                        <xsl:value-of select="attribute::name"/>
                    </xsl:attribute>
                    <xsl:value-of select="attribute::name"/>
                </xsl:element>
            </h3>

            <xsl:call-template name="data.subsection.content"/>
        </xsl:for-each>
    </xsl:template>


    <xsl:template match="SUBSECTION" name="data.subsection.content">
        <table>
            <xsl:for-each select="*">
                <xsl:if test="*">
                    <xsl:call-template name="data.subsubsection.content"/>
                </xsl:if>

                <xsl:if test="not(*)">
                    <tr>
                        <td class="left">
                            <xsl:value-of select="@name"/>
                        </td>
                        <td class="left" width="100%" nowrap="nowrap">

                            <xsl:choose>
                                <xsl:when test="attribute::preformatted">
                                    <pre style="font-size: 10px">
                                       <xsl:value-of select="text()"/>
                                    </pre>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="text()"/>
                                </xsl:otherwise>
                            </xsl:choose>

                        </td>
                    </tr>
                    <xsl:call-template name="create.blank.line"/>
                </xsl:if>
            </xsl:for-each>
        </table>
    </xsl:template>


    <xsl:template match="*" name="data.subsubsection.content">
        <xsl:for-each select="*">
            <tr>
                <td class="left">
                    <xsl:value-of select="@name"/>
                </td>
                <td class="left" width="100%" nowrap="nowrap">

                    <xsl:choose>
                        <xsl:when test="attribute::preformatted">
                            <pre style="font-size: 10px">
                                <xsl:value-of select="text()"/>
                            </pre>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="text()"/>
                        </xsl:otherwise>
                    </xsl:choose>

                </td>
            </tr>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="create.blank.line">
        <xsl:if test="@id = 'licensee'">
            <tr>
                <td colspan="2" class="blank"/>
            </tr>
        </xsl:if>
        <xsl:if test="@id = 'server_version'">
            <tr>
                <td colspan="2" class="blank"/>
            </tr>
        </xsl:if>
        <xsl:if test="@id = 'blank'">
            <tr>
                <td colspan="2" height="20px" class="blank"/>
            </tr>
        </xsl:if>

        <!--<xsl:if test="contains(*, PAGENAME)">
            <tr>
                <td colspan="2" class="blank"/>
            </tr>
        </xsl:if>-->
    </xsl:template>


    <xsl:template match="SECTION/ENTRIES" name="data.stacktrace">
        <div>
        <xsl:for-each select="ENTRIES/ENTRY">
            <h3>
                <xsl:element name="a">
                    <xsl:attribute name="name">stack<xsl:value-of select="position()"/>
                    </xsl:attribute>
                    <xsl:value-of select="MESSAGE"/>
                </xsl:element>
            </h3>
            <div class="text">
                <!--<xsl:value-of select='xs:dateTime("1970-01-01T00:00:00") + $in * xs:dayTimeDuration("PT0.001S")'/>-->
                <xsl:value-of select="LEVEL"/><br/>
                <xsl:value-of select="TIMESTAMP"/> -
                <xsl:value-of select="CLIENTINFO"/>
            </div>
            <h4>Stacktrace (<xsl:value-of select="CRC"/>)</h4>
            <div class="stacktrace">
                <pre>
                    <xsl:value-of disable-output-escaping="no" select="STACKTRACE"/>
                </pre>
            </div>

            <xsl:if test="ADDITIONALINFO">
                <xsl:call-template name="data.additionalinfo">
                    <xsl:with-param name="info.id" select="position()"/>
                </xsl:call-template>
            </xsl:if>

			<xsl:if test="CUSTOM_DESCRIPTION">
				<xsl:call-template name="data.custom_description">
					<xsl:with-param name="info.id" select="position()"/>
				</xsl:call-template>
			</xsl:if>

			<xsl:if test="BINARY">
				<xsl:call-template name="data.binary">
					<xsl:with-param name="info.id" select="position()"/>
				</xsl:call-template>
			</xsl:if>

            <xsl:call-template name="data.backtrace">
                <xsl:with-param name="entry.id" select="position()"/>
            </xsl:call-template>

            <hr color="#d2d2d2"/>

        </xsl:for-each>
        </div>
    </xsl:template>


    <xsl:template match="ADDITIONALINFO" name="data.additionalinfo">
        <xsl:param name="info.id"/>

        <xsl:element name="h4">
            <xsl:attribute name="onclick">toggleDisplay(window.document.getElementById('additionalinfo<xsl:value-of select="$info.id"/>'))</xsl:attribute>
            <xsl:element name="u">Logging</xsl:element>
        </xsl:element>

        <div class="stacktrace">
            <xsl:element name="pre">
                <xsl:attribute name="id">additionalinfo<xsl:value-of select="$info.id"/>
                </xsl:attribute>
                <xsl:attribute name="style">display: none;</xsl:attribute>
                <xsl:value-of disable-output-escaping="no" select="ADDITIONALINFO"/>
            </xsl:element>
        </div>
    </xsl:template>


	<xsl:template match="CUSTOM_DESCRIPTION" name="data.custom_description">
		<xsl:param name="info.id"/>

		<xsl:element name="h4">
			<xsl:attribute name="onclick">toggleDisplay(window.document.getElementById('custom_description<xsl:value-of
					select="$info.id"/>'))
			</xsl:attribute>
			<xsl:element name="u">Custom Description</xsl:element>
		</xsl:element>

		<div class="stacktrace">
			<xsl:element name="pre">
				<xsl:attribute name="id">custom_description<xsl:value-of select="$info.id"/>
				</xsl:attribute>
				<xsl:attribute name="style">display: none;</xsl:attribute>
				<xsl:value-of disable-output-escaping="no" select="CUSTOM_DESCRIPTION"/>
			</xsl:element>
		</div>
	</xsl:template>


	<xsl:template match="BINARY" name="data.binary">
		<xsl:param name="info.id"/>

		<xsl:element name="h4">
			<xsl:attribute name="onclick">toggleDisplay(window.document.getElementById('binary<xsl:value-of
					select="$info.id"/>'))
			</xsl:attribute>
			<xsl:element name="u">Binary</xsl:element>
		</xsl:element>

		<div class="stacktrace">
			<xsl:element name="pre">
				<xsl:attribute name="id">binary<xsl:value-of select="$info.id"/>
				</xsl:attribute>
				<xsl:attribute name="style">display: none;</xsl:attribute>
				<xsl:element name="img">
					<xsl:attribute name="src">
						<xsl:value-of disable-output-escaping="no" select="BINARY"/>
					</xsl:attribute>
					<xsl:attribute name="alt">bin</xsl:attribute>
				</xsl:element>
			</xsl:element>
		</div>
	</xsl:template>


    <xsl:template match="SECTION/THREADDUMPS" name="data.threaddump">
        <div>
        <xsl:for-each select="THREADDUMPS/THREADDUMP">
            <h3>
                <xsl:element name="a">
                    <xsl:attribute name="name">threaddump<xsl:value-of select="position()"/>
                    </xsl:attribute>
                </xsl:element>
            </h3>
            <h4><xsl:value-of select="substring(attribute::name, 1, 250)"/></h4>
            <div class="stacktrace">
                <pre>
                    <xsl:value-of select="text()"/>
                </pre>
            </div>
        </xsl:for-each>
        </div>
    </xsl:template>


    <xsl:template match="ENTRIES" name="data.backtrace">
        <xsl:param name="entry.id"/>
        <xsl:for-each select="BACKTRACE">

            <xsl:element name="h4">
                <xsl:attribute name="onclick">toggleDisplay(window.document.getElementById('backTrace<xsl:value-of select="$entry.id"/>'))</xsl:attribute>
                <xsl:element name="u">Backtrace</xsl:element>
            </xsl:element>

            <xsl:element name="div">
                <xsl:attribute name="id">backTrace<xsl:value-of select="$entry.id"/></xsl:attribute>
                <xsl:attribute name="style">display: none;</xsl:attribute>
                <xsl:attribute name="class">stacktrace</xsl:attribute>
                <xsl:call-template name="data.logbacktrace"/>
                <xsl:call-template name="data.modelchangebacktrace"/>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

    <!-- LOGBACkTRACE -->
    <xsl:template match="LogBackTrace" name="data.logbacktrace">
        <xsl:for-each select="LogBackTrace">
            <table>
                <tr>
                    <td class="left">
                        <b>
                            <xsl:value-of select="name(.)"/>
                        </b>
                    </td>
                    <td class="left">
                        <b>
                            <xsl:value-of select="@name"/>
                        </b>
                    </td>
                </tr>
                <xsl:for-each select="*">
                    <xsl:if test="not(*)">
                        <tr>
                            <td>
                                <xsl:value-of select="local-name()"/>
                            </td>
                            <td>
                                <pre>
                                    <xsl:value-of select="text()"/>
                                </pre>
                            </td>
                        </tr>
                    </xsl:if>
                </xsl:for-each>

                <xsl:call-template name="data.logbacktrace_element"/>
            </table>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="LogBackTraceElement" name="data.logbacktrace_element">
        <xsl:for-each select="LogBackTraceElement">
            <tr>
                <td>
                    <b>
                        <xsl:value-of select="name(.)"/>
                    </b>
                </td>
                <td>
                </td>
            </tr>
            <xsl:for-each select="*">
                <tr>
                    <td>
                        <xsl:value-of select="local-name()"/>
                    </td>
                    <td>
                        <pre>
                            <xsl:value-of select="text()"/>
                        </pre>
                    </td>
                </tr>
            </xsl:for-each>
            <tr>
                <td colspan="2">
                    <hr NOSHADE="1" style="color: 'gray'; height: '1'; text-align: 'left'; width: '100%'"/>
                </td>
            </tr>
        </xsl:for-each>
    </xsl:template>


    <!-- MODELBACkTRACE -->
    <xsl:template match="ModelChangeBackTrace" name="data.modelchangebacktrace">
        <xsl:for-each select="ModelChangeBackTrace">
            <table>
                <tr>
                    <td class="left">
                        <b>
                            <xsl:value-of select="name(.)"/>
                        </b>
                    </td>
                    <td class="left">
                        <b>
                            <xsl:value-of select="@name"/>
                        </b>
                    </td>
                </tr>
                <xsl:for-each select="*">
                    <xsl:if test="not(*)">
                        <tr>
                            <td>
                                <xsl:value-of select="local-name()"/>
                            </td>
                            <td>
                                <pre>
                                    <xsl:value-of select="text()"/>
                                </pre>
                            </td>
                        </tr>
                    </xsl:if>
                </xsl:for-each>

                <xsl:call-template name="data.modelchangetrace_element"/>
            </table>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="ModelChangeTraceElement" name="data.modelchangetrace_element">
        <xsl:for-each select="ModelChangeTraceElement">
            <tr>
                <td>
                    <b>
                        <xsl:value-of select="name(.)"/>
                    </b>
                </td>
                <td>
                </td>
            </tr>
            <xsl:for-each select="*">
                <tr>
                    <td>
                        <xsl:value-of select="local-name()"/>
                    </td>
                    <td>
                        <pre>
                            <xsl:value-of select="text()"/>
                        </pre>
                    </td>
                </tr>
            </xsl:for-each>
            <tr>
                <td colspan="2">
                    <hr NOSHADE="1" style="color: 'gray'; height: '1'; text-align: 'left'; width: '100%'"/>
                </td>
            </tr>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="FOOTER" name="data.footer">
        <br/>
        <br/>
        <div class="text" style="color:#8c8c8c;">
            <xsl:value-of select="FOOTER"/>
        </div>
        <br/>
        <br/>
        <br/>
    </xsl:template>

    <xsl:template name="data.raw">
        <xsl:comment>
            <xsl:value-of select="RAW"/>
        </xsl:comment>
    </xsl:template>

</xsl:stylesheet>