package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.common.base.Logging;
import de.espirit.firstspirit.io.FileHandle;
import de.espirit.firstspirit.io.FileHandleSupport;
import de.espirit.firstspirit.io.FileSystem;
import de.espirit.firstspirit.module.Configuration;
import de.espirit.firstspirit.module.ServerEnvironment;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JComponent;
import java.awt.Frame;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;


/**
 * Configuration of {@link ModuleService}.
 */
public class ModuleServiceConfiguration implements Configuration<ServerEnvironment> {

	private static final Class<ModuleServiceConfiguration> LOGGER = ModuleServiceConfiguration.class;

	private static final String SERVICES_CONFIG = "services.xml";

	private ServerEnvironment _env;
	private ModuleServiceConfigurationComponent _component;
	private List<ServiceConfiguration> _services;


	@Override
	public ServerEnvironment getEnvironment() {
		return _env;
	}


	@Override
	public void init(final String moduleName, final String componentName, final ServerEnvironment env) {
		_env = env;
	}


	@Nullable
	@Override
	public String getParameter(final String name) {
		return null;
	}


	@Override
	public Set<String> getParameterNames() {
		return Collections.emptySet();
	}


	@Override
	public void store() {
		final FileSystem<? extends FileHandle> dir = _env.getConfDir();
		try {
			final FileHandle handle = dir.obtain(SERVICES_CONFIG);
			List<ServiceConfiguration> value = _services;
			if (_component != null) {
				value = _component.getValue();
			}
			if (value == null) {
				value = Collections.emptyList();
			}
			FileHandleSupport.serializeTo(value, handle, getClass().getClassLoader());
		} catch (final IOException e) {
			Logging.logWarning("Failed to store service configuration!", e, LOGGER);
		}
	}


	@Override
	public void load() {
		_services = readServices(_env);
		if (_component != null) {
			_component.setValue(_services);
		}
	}


	/**
	 * Reads and returns the service configuration stored in the configuration filesystem.
	 *
	 * @param env The environment to load the configuration file from.
	 * @return The list of configured services.
	 */
	@NotNull
	static List<ServiceConfiguration> readServices(@NotNull final ServerEnvironment env) {
		final List<ServiceConfiguration> result = new ArrayList<>();
		final FileSystem<? extends FileHandle> dir = env.getConfDir();
		try {
			final FileHandle handle = dir.obtain(SERVICES_CONFIG);
			final Object services = FileHandleSupport.deserializeFrom(handle, ServiceConfiguration.class.getClassLoader());
			if (services instanceof Iterable<?>) {
				for (final Object entry : (Iterable<?>) services) {
					if (entry instanceof ServiceConfiguration) {
						result.add((ServiceConfiguration) entry);
					}
				}
			}
		} catch (final IOException e) {
			Logging.logWarning("Failed to store service configuration!", e, LOGGER);
		}
		return result;
	}


	@Nullable
	@Override
	public JComponent getGui(final Frame masterFrame) {
		if (_component == null) {
			List<ServiceConfiguration> value = _services;
			if (value == null) {
				value = Collections.emptyList();
			}
			_component = new ModuleServiceConfigurationComponent(value);
		}
		return _component;
	}


	@Override
	public boolean hasGui() {
		return true;
	}
}
