package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.common.tools.Windows;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ParameterBoolean;
import de.espirit.firstspirit.client.plugin.report.ParameterText;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.JTextComponent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;


/**
 * UI for the {@link ModuleServiceConfiguration}.
 */
public class ModuleServiceConfigurationComponent extends JPanel implements MouseListener, ListSelectionListener {

	private static final long serialVersionUID = 1L;

	private final DefaultListModel<ServiceConfiguration> _model;
	private final JList<ServiceConfiguration> _services;
	private final JButton _edit;
	private final JButton _remove;


	public ModuleServiceConfigurationComponent(@NotNull final List<ServiceConfiguration> value) {
		setLayout(new BorderLayout());

		_model = new DefaultListModel<>();
		setValue(value);

		final JLabel listTitle = new JLabel("SOAP Services");
		listTitle.setForeground(Color.GRAY);
		listTitle.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
		add(listTitle, BorderLayout.NORTH);

		_services = new JList<>(_model);
		_services.enableInputMethods(true);
		_services.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		_services.addListSelectionListener(this);
		_services.addMouseListener(this);
		add(new JScrollPane(_services), BorderLayout.CENTER);

		final JPanel buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
		add(buttons, BorderLayout.SOUTH);

		final JButton add = new JButton("Add");
		add.addActionListener(e -> {
			final ServiceConfiguration configuration = new ServiceConfiguration();
			if (configure(configuration)) {
				_model.addElement(configuration);
				_services.repaint();
			}
		});
		buttons.add(add);

		_edit = new JButton("Edit");
		_edit.setEnabled(false);
		_edit.addActionListener(e -> {
			if (configure(_services.getSelectedValue())) {
				_services.repaint();
			}
		});
		buttons.add(_edit);

		_remove = new JButton("Remove");
		_remove.setEnabled(false);
		_remove.addActionListener(e -> {
			final ServiceConfiguration selection = _services.getSelectedValue();
			if (selection != null) {
				if (_model.removeElement(selection)) {
					_services.repaint();
				}
			}
		});
		buttons.add(_remove);
	}


	/**
	 * Set the currently configured services.
	 *
	 * @param value The configured services.
	 */
	public void setValue(@NotNull final List<ServiceConfiguration> value) {
		_model.clear();
		for (final ServiceConfiguration configuration : value) {
			_model.addElement(configuration);
		}
	}


	/**
	 * Returns the currently configured services.
	 *
	 * @return The configured services.
	 */
	@NotNull
	public List<ServiceConfiguration> getValue() {
		final List<ServiceConfiguration> value = new ArrayList<>();
		for (int i = 0; i < _model.getSize(); i++) {
			value.add(_model.getElementAt(i));
		}
		return value;
	}


	/**
	 * Open configuration dialog for the specified service.
	 *
	 * @param configuration The service to configure.
	 * @return Whether or not the service configuration has been modified.
	 */
	private boolean configure(@Nullable final ServiceConfiguration configuration) {
		return configuration != null && new ServiceConfigurationDialog(configuration).perform();
	}


	@Override
	public void valueChanged(final ListSelectionEvent e) {
		_edit.setEnabled(_services.getSelectedValue() != null);
		_remove.setEnabled(_services.getSelectedValue() != null);
	}


	@Override
	public void mouseClicked(final MouseEvent e) {
		if (e.getClickCount() == 2) {
			if (configure(_services.getSelectedValue())) {
				_services.repaint();
			}
		}
	}


	@Override
	public void mousePressed(final MouseEvent e) {
	}


	@Override
	public void mouseReleased(final MouseEvent e) {
	}


	@Override
	public void mouseEntered(final MouseEvent e) {
	}


	@Override
	public void mouseExited(final MouseEvent e) {
	}


	/**
	 * Dialog for the configuration of one service.
	 */
	private class ServiceConfigurationDialog extends JDialog implements DocumentListener {

		private static final long serialVersionUID = 1L;

		private final ServiceConfiguration _value;
		private final JTextField _name;
		private final JTextField _wsdl;
		private final JTextField _endpoint;
		private final JTextField _binding;
		private final JTextField _byId_operation;
		private final JTextField _byId_idParameter;
		private final JTextArea _byId_snippetHeader;
		private final JTextArea _byId_snippetExtract;
		private final JTextArea _byId_snippetImage;
		private final JTextField _list_operation;
		private final JTextField _list_entry;
		private final JTextField _list_entryId;
		private final JButton _save;

		private boolean _saveRequest;
		private final DefaultListModel<ServiceConfiguration.ParameterConfiguration> _parameterModel;


		ServiceConfigurationDialog(@NotNull final ServiceConfiguration value) {
			setModal(true);
			setTitle("SOAP Service");
			_value = value;

			final JPanel content = new JPanel();

			final TableLayout layout = new TableLayout();
			layout.setColumn(new double[] {TableLayout.PREFERRED, TableLayout.FILL});
			layout.setHGap(5);
			layout.setVGap(5);
			content.setLayout(layout);
			content.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			setContentPane(content);

			int row = -1;

			layout.insertRow(++row, TableLayout.PREFERRED);
			final JLabel listTitle = new JLabel("Service Definition");
			listTitle.setForeground(Color.GRAY);
			listTitle.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
			content.add(listTitle, new TableLayoutConstraints(0, row, 1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Name"), new TableLayoutConstraints(0, row));
			_name = new JTextField(_value.serviceName);
			_name.getDocument().addDocumentListener(this);
			content.add(_name, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("WSDL URI"), new TableLayoutConstraints(0, row));
			_wsdl = new JTextField(_value.wsdlUrl);
			_wsdl.getDocument().addDocumentListener(this);
			content.add(_wsdl, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Endpoint URI"), new TableLayoutConstraints(0, row));
			_endpoint = new JTextField(_value.endpoint);
			_endpoint.getDocument().addDocumentListener(this);
			content.add(_endpoint, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Binding Name"), new TableLayoutConstraints(0, row));
			_binding = new JTextField(_value.binding);
			_binding.getDocument().addDocumentListener(this);
			content.add(_binding, new TableLayoutConstraints(1, row));

			// Operation getById

			layout.insertRow(++row, TableLayout.PREFERRED);
			final JLabel byIdTitle = new JLabel("Operation: Object by ID");
			byIdTitle.setForeground(Color.GRAY);
			byIdTitle.setBorder(BorderFactory.createEmptyBorder(20, 0, 10, 0));
			content.add(byIdTitle, new TableLayoutConstraints(0, row, 1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Name"), new TableLayoutConstraints(0, row));
			_byId_operation = new JTextField(_value.operationById);
			_byId_operation.getDocument().addDocumentListener(this);
			content.add(_byId_operation, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("ID Parameter (Expression)"), new TableLayoutConstraints(0, row));
			_byId_idParameter = new JTextField(_value.operationById_evaluateSetId);
			_byId_idParameter.getDocument().addDocumentListener(this);
			content.add(_byId_idParameter, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Snippet Header"), new TableLayoutConstraints(0, row));
			_byId_snippetHeader = new JTextArea(_value.operationById_evaluateHeader);
			_byId_snippetHeader.setRows(2);
			_byId_snippetHeader.setLineWrap(true);
			_byId_snippetHeader.getDocument().addDocumentListener(this);
			content.add(new JScrollPane(_byId_snippetHeader), new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Snippet Extract"), new TableLayoutConstraints(0, row));
			_byId_snippetExtract = new JTextArea(_value.operationById_evaluateExtract);
			_byId_snippetExtract.setRows(2);
			_byId_snippetExtract.setLineWrap(true);
			_byId_snippetExtract.getDocument().addDocumentListener(this);
			content.add(new JScrollPane(_byId_snippetExtract), new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Snippet Image URI"), new TableLayoutConstraints(0, row));
			_byId_snippetImage = new JTextArea(_value.operationById_evaluateImage);
			_byId_snippetImage.setRows(2);
			_byId_snippetImage.setLineWrap(true);
			_byId_snippetImage.getDocument().addDocumentListener(this);
			content.add(new JScrollPane(_byId_snippetImage), new TableLayoutConstraints(1, row));

			// Operation getList

			layout.insertRow(++row, TableLayout.PREFERRED);
			final JLabel serviceTitle = new JLabel("Operation: List Objects");
			serviceTitle.setForeground(Color.GRAY);
			serviceTitle.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
			content.add(serviceTitle, new TableLayoutConstraints(0, row, 1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Name"), new TableLayoutConstraints(0, row));
			_list_operation = new JTextField(_value.operationList);
			_list_operation.getDocument().addDocumentListener(this);
			content.add(_list_operation, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Result Entry (Expression)"), new TableLayoutConstraints(0, row));
			_list_entry = new JTextField(_value.operationList_evaluateEntry);
			_list_entry.getDocument().addDocumentListener(this);
			content.add(_list_entry, new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Result Entry ID (Expression)"), new TableLayoutConstraints(0, row));
			_list_entryId = new JTextField(_value.operationList_evaluateEntryId);
			_list_entryId.getDocument().addDocumentListener(this);
			content.add(_list_entryId, new TableLayoutConstraints(1, row));

			// Operation getList: Request parameter configuration

			layout.insertRow(++row, TableLayout.PREFERRED);
			content.add(new JLabel("Request Parameter"), new TableLayoutConstraints(0, row));

			final JButton parameterAdd = new JButton("Add");
			final JButton parameterEdit = new JButton("Edit");
			final JButton parameterRemove = new JButton("Remove");

			_parameterModel = new DefaultListModel<>();
			final JList<ServiceConfiguration.ParameterConfiguration> parameter = new JList<>(_parameterModel);
			parameter.enableInputMethods(true);
			parameter.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			parameter.addListSelectionListener(e -> {
				parameterEdit.setEnabled(parameter.getSelectedValue() != null);
				parameterRemove.setEnabled(parameter.getSelectedValue() != null);
			});
			parameter.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseClicked(final MouseEvent e) {
					if (e.getClickCount() == 2) {
						if (configure(parameter.getSelectedValue())) {
							onServiceModification();
							parameter.repaint();
						}
					}
				}
			});
			content.add(new JScrollPane(parameter), new TableLayoutConstraints(1, row));

			layout.insertRow(++row, TableLayout.PREFERRED);
			final JPanel parameterButtons = new JPanel(new FlowLayout(FlowLayout.CENTER));
			content.add(parameterButtons, new TableLayoutConstraints(1, row));

			parameterAdd.addActionListener(e -> {
				final ServiceConfiguration.ParameterConfiguration configuration = new ServiceConfiguration.ParameterConfiguration();
				if (configure(configuration)) {
					_parameterModel.addElement(configuration);
					onServiceModification();
					parameter.repaint();
				}
			});
			parameterButtons.add(parameterAdd);

			parameterEdit.setEnabled(false);
			parameterEdit.addActionListener(e -> {
				if (configure(parameter.getSelectedValue())) {
					onServiceModification();
					parameter.repaint();
				}
			});
			parameterButtons.add(parameterEdit);

			parameterRemove.setEnabled(false);
			parameterRemove.addActionListener(e -> {
				final ServiceConfiguration.ParameterConfiguration selection = parameter.getSelectedValue();
				if (selection != null) {
					if (_parameterModel.removeElement(selection)) {
						onServiceModification();
						parameter.repaint();
					}
				}
			});
			parameterButtons.add(parameterRemove);

			// Controlling buttons

			layout.insertRow(++row, TableLayout.FILL);

			layout.insertRow(++row, TableLayout.PREFERRED);
			final JPanel buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
			content.add(buttons, new TableLayoutConstraints(0, row, 1, row));

			_save = new JButton("Save");
			_save.setEnabled(false);
			_save.addActionListener(e -> {
				_saveRequest = true;
				setVisible(false);
			});
			buttons.add(_save);

			final JButton cancel = new JButton("Cancel");
			cancel.addActionListener(e -> setVisible(false));
			buttons.add(cancel);

			pack();
		}


		/**
		 * Shows the configuration dialog, and waits for its closing.
		 *
		 * @return Whether or not the configuration has been modified.
		 */
		public boolean perform() {
			Windows.centerWindow(this);
			setVisible(true);
			if (_saveRequest && canSave()) {
				save();
				return true;
			}
			return false;
		}


		/**
		 * Opens a configuration dialog for the specified parameter.
		 *
		 * @param configuration The parameter to configure.
		 * @return Whether or not the configuration has been specified.
		 */
		private boolean configure(@Nullable final ServiceConfiguration.ParameterConfiguration configuration) {
			return configuration != null && new RequestParameterDialog(configuration).perform();
		}


		/**
		 * Checks whether or not the current modifications can be saved.
		 *
		 * @return {@code true} if the changes can be saved, {@code false} otherwise.
		 */
		public boolean canSave() {
			final String serviceName = _name.getText();
			if (serviceName == null || serviceName.isEmpty()) {
				return false;
			}
			for (int i = 0; i < _model.getSize(); i++) {
				final ServiceConfiguration configuration = _model.getElementAt(i);
				if (!_value.equals(configuration) && serviceName.equals(configuration.serviceName)) {
					return false;
				}
			}
			return true;
		}


		/**
		 * Save the changes made in the service configuration.
		 */
		public void save() {
			_value.serviceName = getValue(_name);
			_value.wsdlUrl = getValue(_wsdl);
			_value.endpoint = getValue(_endpoint);
			_value.binding = getValue(_binding);

			_value.operationById = _byId_operation.getText();
			_value.operationById_evaluateSetId = getValue(_byId_idParameter);
			_value.operationById_evaluateHeader = getValue(_byId_snippetHeader);
			_value.operationById_evaluateExtract = getValue(_byId_snippetExtract);
			_value.operationById_evaluateImage = getValue(_byId_snippetImage);

			_value.operationList = getValue(_list_operation);
			_value.operationList_evaluateEntry = getValue(_list_entry);
			_value.operationList_evaluateEntryId = getValue(_list_entryId);
			final List<ServiceConfiguration.ParameterConfiguration> parameter = new ArrayList<>();
			for (int i = 0; i < _parameterModel.getSize(); i++) {
				parameter.add(_parameterModel.getElementAt(i));
			}
			_value.operationList_parameter = parameter;
		}


		/**
		 * Returns the value of the specified text component.
		 * This method will trim the content and return {@code null} for empty strings.
		 *
		 * @param component The component to return the value for.
		 * @return The component's value.
		 */
		@Nullable
		private String getValue(@NotNull final JTextComponent component) {
			return getValue(component, true);
		}


		/**
		 * Returns the value of the specified text component.
		 * This method will return {@code null} for empty strings.
		 *
		 * @param component The component to return the value for.
		 * @return The component's value.
		 */
		@Nullable
		private String getValue(@NotNull final JTextComponent component, final boolean trim) {
			String text = component.getText();
			if (text != null) {
				if (trim) {
					text = text.trim();
				}
				if (!text.isEmpty()) {
					return text;
				}
			}
			return null;
		}


		@Override
		public void insertUpdate(final DocumentEvent e) {
			onServiceModification();
		}


		@Override
		public void removeUpdate(final DocumentEvent e) {
			onServiceModification();
		}


		@Override
		public void changedUpdate(final DocumentEvent e) {
			onServiceModification();
		}


		/**
		 * Notification about a modification in the current configuration.
		 */
		private void onServiceModification() {
			_save.setEnabled(canSave());
		}


		/**
		 * Dialog for the configration of a parameter.
		 */
		private class RequestParameterDialog extends JDialog implements DocumentListener {

			private static final long serialVersionUID = 1L;

			private final ServiceConfiguration.ParameterConfiguration _value;
			private final JComboBox<Class<? extends Parameter<?>>> _type;
			private final JTextField _name;
			private final JTextField _label;
			private final JTextField _defaultValueText;
			private final JCheckBox _defaultValueBoolean;
			private final JTextField _expression;
			private final JButton _save;

			private boolean _saveRequest;


			RequestParameterDialog(final ServiceConfiguration.ParameterConfiguration value) {
				setModal(true);
				setTitle("Request Parameter");
				_value = value;

				final JPanel content = new JPanel();

				final TableLayout layout = new TableLayout();
				layout.setColumn(new double[]{TableLayout.PREFERRED, TableLayout.FILL});
				layout.setHGap(5);
				layout.setVGap(5);
				content.setLayout(layout);
				content.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
				setContentPane(content);

				int row = -1;

				String name = null;
				String label = null;
				String defaultValueText = null;
				Boolean defaultValueBoolean = null;
				if (_value.parameter != null) {
					name = _value.parameter.getName();
					if (_value.parameter instanceof ParameterText) {
						label = ((ParameterText) _value.parameter).getPlaceholder();
						defaultValueText = ((ParameterText) _value.parameter).getDefaultValue();
					} else if (_value.parameter instanceof ParameterBoolean) {
						label = ((ParameterBoolean) _value.parameter).getLabel();
						defaultValueBoolean = ((ParameterBoolean) _value.parameter).getDefaultValue();
					}
				}

				layout.insertRow(++row, TableLayout.PREFERRED);
				content.add(new JLabel("Type"), new TableLayoutConstraints(0, row));
				_type = new JComboBox<>();
				_type.addItem(ParameterText.class);
				_type.addItem(ParameterBoolean.class);
				_type.addItemListener(new ItemListener() {
					@SuppressWarnings("unchecked")
					@Override
					public void itemStateChanged(final ItemEvent e) {
						final Class<? extends Parameter<?>> selection = (Class<? extends Parameter<?>>) _type.getSelectedItem();
						_defaultValueText.setVisible(ParameterText.class.equals(selection));
						_defaultValueBoolean.setVisible(ParameterBoolean.class.equals(selection));
					}
				});
				_type.setRenderer(new DefaultListCellRenderer() {
					private static final long serialVersionUID = 1L;
					@SuppressWarnings("unchecked")
					@Override
					public Component getListCellRendererComponent(final JList<?> list, final Object value, final int index, final boolean isSelected, final boolean cellHasFocus) {
						super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
						final Class<? extends Parameter<?>> type = (Class<? extends Parameter<?>>) value;
						if (ParameterText.class.equals(type)) {
							setText("Text");
						} else if (ParameterBoolean.class.equals(type)) {
							setText("Boolean");
						} else {
							setText("<unknown>");
						}
						return this;
					}
				});
				content.add(_type, new TableLayoutConstraints(1, row));

				layout.insertRow(++row, TableLayout.PREFERRED);
				content.add(new JLabel("Name"), new TableLayoutConstraints(0, row));
				_name = new JTextField(name);
				_name.getDocument().addDocumentListener(this);
				content.add(_name, new TableLayoutConstraints(1, row));

				layout.insertRow(++row, TableLayout.PREFERRED);
				content.add(new JLabel("Label"), new TableLayoutConstraints(0, row));
				_label = new JTextField(label);
				_label.getDocument().addDocumentListener(this);
				content.add(_label, new TableLayoutConstraints(1, row));

				layout.insertRow(++row, TableLayout.PREFERRED);
				content.add(new JLabel("Default Value"), new TableLayoutConstraints(0, row));

				_defaultValueText = new JTextField(defaultValueText);
				_defaultValueText.getDocument().addDocumentListener(this);
				content.add(_defaultValueText, new TableLayoutConstraints(1, row));

				_defaultValueBoolean = new JCheckBox();
				_defaultValueBoolean.setSelected(Boolean.TRUE.equals(defaultValueBoolean));
				_defaultValueBoolean.addItemListener(e -> onParameterModification());
				content.add(_defaultValueBoolean, new TableLayoutConstraints(1, row));

				layout.insertRow(++row, TableLayout.PREFERRED);
				content.add(new JLabel("Expression"), new TableLayoutConstraints(0, row));
				_expression = new JTextField(_value.evaluateSetValue);
				_expression.getDocument().addDocumentListener(this);
				content.add(_expression, new TableLayoutConstraints(1, row));

				// Controlling buttons

				layout.insertRow(++row, TableLayout.FILL);

				layout.insertRow(++row, TableLayout.PREFERRED);
				final JPanel buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
				content.add(buttons, new TableLayoutConstraints(0, row, 1, row));

				_save = new JButton("Save");
				_save.setEnabled(false);
				_save.addActionListener(e -> {
					_saveRequest = true;
					setVisible(false);
				});
				buttons.add(_save);

				final JButton cancel = new JButton("Cancel");
				cancel.addActionListener(e -> setVisible(false));
				buttons.add(cancel);

				setMinimumSize(new Dimension(300, 0));
				pack();
			}


			/**
			 * Shows the configuration dialog, and waits for its closing.
			 *
			 * @return Whether or not the configuration has been modified.
			 */
			public boolean perform() {
				Windows.centerWindow(this);
				setVisible(true);
				if (_saveRequest && canSave()) {
					save();
					return true;
				}
				return false;
			}


			/**
			 * Checks whether or not the current modifications can be saved.
			 *
			 * @return {@code true} if the changes can be saved, {@code false} otherwise.
			 */
			public boolean canSave() {
				final String name = _name.getText();
				if (name == null || name.isEmpty()) {
					return false;
				}
				for (int i = 0; i < _parameterModel.getSize(); i++) {
					final ServiceConfiguration.ParameterConfiguration configuration = _parameterModel.getElementAt(i);
					if (!_value.equals(configuration) && configuration.parameter != null && name.equals(configuration.parameter.getName())) {
						return false;
					}
				}
				return true;
			}


			/**
			 * Save the changes made in the parameter configuration.
			 */
			@SuppressWarnings("unchecked")
			public void save() {
				final Class<? extends Parameter<?>> type = (Class<? extends Parameter<?>>) _type.getSelectedItem();
				if (ParameterText.class.equals(type)) {
					_value.parameter = Parameter.Factory.createText(_name.getText(), _label.getText(), _defaultValueText.getText());
				} else if (ParameterBoolean.class.equals(type)) {
					_value.parameter = Parameter.Factory.createBoolean(_name.getText(), _label.getText(), _defaultValueBoolean.isSelected());
				} else {
					_value.parameter = null;
				}
				_value.evaluateSetValue = _expression.getText();
			}


			@Override
			public void insertUpdate(final DocumentEvent e) {
				onParameterModification();
			}


			@Override
			public void removeUpdate(final DocumentEvent e) {
				onParameterModification();
			}


			@Override
			public void changedUpdate(final DocumentEvent e) {
				onParameterModification();
			}



			/**
			 * Notification about a modification in the current configuration.
			 */
			private void onParameterModification() {
				_save.setEnabled(canSave());
			}
		}
	}
}
