package com.espirit.core.samples.dataaccessplugin.soap;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * Provides access to properties of the implementing object.
 */
public interface PropertyAccessor {

	/**
	 * Returns the value of the property associated with the given name.
	 *
	 * @param name The name of the property to return.
	 * @return The property's value, or {@code null}.
	 * @throws IllegalStateException If accessing the requested member failed for some reason.
	 */
	@Nullable
	Object getProperty(@NotNull String name) throws IllegalStateException;

	/**
	 * Set the value of the specified property.
	 *
	 * @param name The name of the property to set.
	 * @param value The property's new value.
	 * @throws IllegalStateException If accessing the requested member failed for some reason.
	 */
	void setProperty(@NotNull String name, @Nullable Object value) throws IllegalStateException;
}
