package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.common.GomDoc;
import de.espirit.firstspirit.access.ServicesBroker;
import de.espirit.firstspirit.access.store.templatestore.gom.AbstractGomElement;
import de.espirit.firstspirit.access.store.templatestore.gom.GomCheckable;
import de.espirit.firstspirit.access.store.templatestore.gom.GomValidationError;


/**
 * GOM configuration of the {@link SOAPDataAccessPlugin}.
 */
public class SOAPDataAccessConfiguration extends AbstractGomElement implements GomCheckable {

	private static final long serialVersionUID = 1L;

	private String _serviceName;


	@Override
	protected String getDefaultTag() {
		return "SOAP_DATAACCESS_CONFIGURATION";
	}


	@GomDoc(description="Name of the SOAP service configuration to use.", since="5.2")
	public String getServiceName() {
		return _serviceName;
	}


	@SuppressWarnings("unused")
	public void setServiceName(final String serviceName) {
		_serviceName = serviceName;
	}


	@Override
	public void verify() throws IllegalStateException {
		if (_serviceName == null) {
			throw new IllegalStateException("ServiceName must be defined.");
		}
	}


	@Override
	public void validate(final Context context) throws GomValidationError {
		verify();
		final ServicesBroker services = context.requireSpecialist(ServicesBroker.TYPE);
		final ModuleService moduleService = services.getService(ModuleService.class);
		final String serviceName = getServiceName();
		final ServiceConfiguration configuration = moduleService.getConfiguration(serviceName);
		if (configuration == null) {
			throw new  GomValidationError("There is no service '" + serviceName + "'", this, null);
		}
	}
}
