package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.DataSnippetProvider;
import de.espirit.firstspirit.client.plugin.dataaccess.DataStreamBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.SessionAspectType;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.NoSuchElementException;


public class SOAPDataAccessSession implements DataAccessSession<SOAPObject> {

	private final BaseContext _context;
	private final ServiceClient _service;


	public SOAPDataAccessSession(@NotNull final BaseContext context, @NotNull final ServiceClient service) {
		_context = context;
		_service = service;
	}


	@Nullable
	@Override
	public <A> A getAspect(@NotNull final SessionAspectType<A> type) {
		return null;
	}


	@NotNull
	@Override
	public SOAPObject getData(@NotNull final String identifier) throws NoSuchElementException {
		final SOAPObject object = _service.getObjectById(identifier);
		if (object != null) {
			return object;
		}
		throw new NoSuchElementException("Unknown object with identifier '" + identifier + "'!");
	}


	@NotNull
	@Override
	public List<SOAPObject> getData(@NotNull final Collection<String> identifiers) {
		final List<SOAPObject> list = new ArrayList<>(identifiers.size());
		for (final String identifier : identifiers) {
			list.add(_service.getObjectById(identifier));
		}
		return list;
	}


	@NotNull
	@Override
	public String getIdentifier(@NotNull final SOAPObject object) throws NoSuchElementException {
		return object.getId();
	}


	@NotNull
	@Override
	public DataSnippetProvider<SOAPObject> createDataSnippetProvider() {
		return new SOAPDataSnippetProvider(_context);
	}


	@NotNull
	@Override
	public DataStreamBuilder<SOAPObject> createDataStreamBuilder() {
		return new SOAPDataStreamBuilder(_service);
	}
}
