package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.ServicesBroker;
import de.espirit.firstspirit.access.store.templatestore.gom.GomElement;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSessionBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.GomConfigurable;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.SessionBuilderAspectMap;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.SessionBuilderAspectType;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


public class SOAPDataAccessSessionBuilder implements DataAccessSessionBuilder<SOAPObject>, GomConfigurable {

	private final SessionBuilderAspectMap _aspects = new SessionBuilderAspectMap();
	private GomElement _gom;


	public SOAPDataAccessSessionBuilder() {
		_aspects.put(GomConfigurable.TYPE, this);
	}


	@NotNull
	@Override
	public GomElement createConfiguration() {
		return new SOAPDataAccessConfiguration();
	}


	@Override
	public void setConfiguration(@NotNull final GomElement configuration) {
		_gom = configuration;
	}


	@Nullable
	@Override
	public <A> A getAspect(@NotNull final SessionBuilderAspectType<A> type) {
		return _aspects.get(type);
	}


	@NotNull
	@Override
	public DataAccessSession<SOAPObject> createSession(@NotNull final BaseContext context) {
		final SOAPDataAccessConfiguration gom = (SOAPDataAccessConfiguration) _gom;
		final String serviceName = gom.getServiceName();
		final ServicesBroker services = context.requireSpecialist(ServicesBroker.TYPE);
		final ModuleService moduleService = services.getService(ModuleService.class);
		final ServiceConfiguration configuration = moduleService.getConfiguration(serviceName);
		if (configuration == null) {
			throw new IllegalStateException("Unknown configuration " + serviceName);
		}
		final ServiceClient service = new ServiceClient(configuration);
		return new SOAPDataAccessSession(context, service);
	}
}
