package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.firstspirit.client.plugin.dataaccess.DataStream;
import de.espirit.firstspirit.client.plugin.report.ParameterMap;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;


public class SOAPDataStream implements DataStream<SOAPObject> {

	private final ServiceClient _service;
	private final ParameterMap _filter;

	private ConcurrentLinkedQueue<SOAPObject> _queue;
	private int _total;


	public SOAPDataStream(@NotNull final ServiceClient service, @Nullable final ParameterMap filter) {
		_service = service;
		_filter = filter;
	}


	@NotNull
	@Override
	public List<SOAPObject> getNext(final int count) {
		ensureQueue();
		final List<SOAPObject> result = new ArrayList<>();
		while (result.size() < count && !_queue.isEmpty()) {
			result.add(_queue.poll());
		}
		return result;
	}


	@Override
	public boolean hasNext() {
		ensureQueue();
		return !_queue.isEmpty();
	}


	@Override
	public int getTotal() {
		return _total;
	}


	@Override
	public void close() {
	}


	private void ensureQueue() {
		if (_queue == null) {
			final List<SOAPObject> list = _service.getObjects(_filter);
			_queue = new ConcurrentLinkedQueue<>(list);
			_total = _queue.size();
		}
	}
}
