package com.espirit.core.samples.dataaccessplugin.soap;

import de.espirit.firstspirit.client.plugin.dataaccess.DataStream;
import de.espirit.firstspirit.client.plugin.dataaccess.DataStreamBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Filterable;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.StreamBuilderAspectMap;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.StreamBuilderAspectType;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ParameterMap;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;


public class SOAPDataStreamBuilder implements DataStreamBuilder<SOAPObject>, Filterable {

	private final ServiceClient _service;
	private final StreamBuilderAspectMap _aspects = new StreamBuilderAspectMap();

	private ParameterMap _filter;


	public SOAPDataStreamBuilder(@NotNull final ServiceClient service) {
		_service = service;
		_aspects.put(Filterable.TYPE, this);
	}


	@NotNull
	@Override
	public List<Parameter<?>> getDefinedParameters() {
		final List<Parameter<?>> parameters = new ArrayList<>();
		final ServiceConfiguration configuration = _service.getConfiguration();
		final List<ServiceConfiguration.ParameterConfiguration> operationList_parameter = configuration.operationList_parameter;
		if (operationList_parameter != null) {
			for (final ServiceConfiguration.ParameterConfiguration entry : operationList_parameter) {
				parameters.add(entry.parameter);
			}
		}
		return parameters;
	}


	@Override
	public void setFilter(@NotNull final ParameterMap filter) {
		_filter = filter;
	}


	@Nullable
	@Override
	public <A> A getAspect(@NotNull final StreamBuilderAspectType<A> type) {
		return _aspects.get(type);
	}


	@NotNull
	@Override
	public DataStream<SOAPObject> createDataStream() {
		return new SOAPDataStream(_service, _filter);
	}
}
